<?php
/**
 * Copyright (C) 2022  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\Areas\DataProcessor\Source;

use Civi\DataProcessor\DataSpecification\DataSpecification;
use Civi\DataProcessor\DataSpecification\FieldSpecification;
use Civi\DataProcessor\Source\AbstractCivicrmEntitySource;
use CRM_Areas_ExtensionUtil as E;

class AreaDefinitionDataSource extends AbstractCivicrmEntitySource {


  /**
   * Returns the table name of this entity
   *
   * @return String
   */
  protected function getTable() {
    return 'civicrm_area_definition';
  }

  /**
   * Returns the entity name
   *
   * @return String
   */
  protected function getEntity() {
    return 'AreaDefinition';
  }

  /**
   * Load the fields from this entity.
   *
   * @param DataSpecification $dataSpecification
   * @param array $fieldsToSkip
   * @throws \Civi\DataProcessor\DataSpecification\FieldExistsException
   */
  protected function loadFields(DataSpecification $dataSpecification, $fieldsToSkip=array()) {
    $aliasPrefix = '';
    if ($this->getSourceName()) {
      $aliasPrefix = $this->getSourceName() . '_';
    }
    $countryOptions = \CRM_Core_BAO_Address::buildOptions('country_id');
    $stateProvinceOptions = \CRM_Core_BAO_Address::buildOptions('state_province_id');
    $fieldSpec = new FieldSpecification('id', 'Integer', E::ts('ID'), null, $aliasPrefix.'id');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
    $fieldSpec = new FieldSpecification('area_id', 'Integer', E::ts('Area ID'), null, $aliasPrefix.'area_id');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
    $fieldSpec = new FieldSpecification('type', 'String', E::ts('Type'), civicrm_area_definition_types(), $aliasPrefix.'type');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
    $fieldSpec = new FieldSpecification('country_id', 'Integer', E::ts('Country ID'), $countryOptions, $aliasPrefix.'country_id');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
    $fieldSpec = new FieldSpecification('state_province_id', 'Integer', E::ts('State/Province ID'), $stateProvinceOptions, $aliasPrefix.'state_province_id');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
    $fieldSpec = new FieldSpecification('city', 'String', E::ts('City'), null, $aliasPrefix.'city');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
    $fieldSpec = new FieldSpecification('postal_code', 'String', E::ts('Postal Code'), null, $aliasPrefix.'postal_code');
    $dataSpecification->addFieldSpecification($fieldSpec->name, $fieldSpec);
  }

  /**
   * Add custom fields to the available fields section
   *
   * @param DataSpecification $dataSpecification
   * @param bool $onlySearchAbleFields
   * @param $entity
   */
  protected function loadCustomGroupsAndFields(DataSpecification $dataSpecification, $onlySearchAbleFields, $entity=null) {
    // Do nothing
  }


}
